//$Id: client.h,v 1.5 1997/11/30 12:16:42 parallax Exp $
//class Client actually handles the guts of IRC parsing

#ifndef CLIENT_H
#define CLIENT_H

#include <qlist.h>
#include <qobject.h>
#include <qsocknot.h>
#include <qstring.h>

#include "dcc.h"
#include "modules.h"

//KDE
#include <kprocess.h>

class BanRequest
{
protected:
    QString nick;
    QString channel;
    QString reason;
    int type;
public:
    enum { NORMAL, HOST, DOMAIN, NICK };
    BanRequest(const char *n, const char *c, const char *r=0, int t=NORMAL) { nick = n; channel = c; reason = r; type = t;}
    const char *Nick() { return nick; }
    const char *Channel() { return channel; }
    const char *Reason() { return reason; }
    int Type() { return type; }
};

class Client : public QObject {
    Q_OBJECT

public: /* Procedures */
    virtual ~Client(); 
    Client(QObject *parent=0, const char *name=0);
    void ConnectClient(int fd); // connects the client to the server connection
    bool Connected() { return connected; }
    static int ConnectToServer(const char *server, int port);
    bool DisconnectFromServer(const char *msg);
    static const char *LocalAddress() { return localAddress; }
    void ParseServer(const char *data);
    void ParseUser(const char *data, const char *win);
    bool Registered() { return registered; }
    bool SendToServer(const char *buff);
    QString UnixTimeToDate ( const char *);
    const char *Version() { return version; }
    void Output(int type, const char *window, const char *txt);   

protected slots:
    void ReadData(); // read data from the socket
    void WriteData(); // write data to the socket

protected: /* Procedures */
    void ParseNumeric( int, char*, char*);
    void ParseWord( char*, char*, char*);
    void HandleEvent(int type, const char *arg1, const char *arg2=0);
    void HandlePrivmsg( char *, char *, char *, char *);
    void HandleMode( char *, char *);
    bool HandleBuiltInCommand ( const char *cmd, const char *rest, const char *win);

protected: /* Data */
    QSocketNotifier *rsn, *wsn; //socket notifiers make our job MUCH easier   
    bool registered;
    modLoader kircModules;

public:
    bool connected; // holds connnection state
    QString incomplete; // sometimes lag means we don't get the whole thing
    QString serverName;
    QString serverPort;
    QString version;
    static QString localAddress;
    QList<BanRequest> *kickBanList;

//located in dcc.cpp
public:
    void CloseDCCChat(const char *nick);
    void SendToDCCChat(const char *nick, const char *buff);
protected:
    void DoDCC(const char *cmd);
    void DCCSend(const char *nick, const char *file);
    void DCCGet(const char *nick);
    void DCCChat(const char *nick);
    int CreateDCCPassiveConnection(int &port); // creates a listening socket, returns fd, fills in the value for port
    QList <KProcess> *dccProcs;
    QList <DCCOffer> *dccOffers;
    QList <DCCChatSession> *dccChats;
    QList <KProcess> *processes;
    QList <QString> *procNames;
    QSocketNotifier *dccChatRead;//, *dccChatWrite;    
protected slots:
    void DCCProcessExited(KProcess *);
    void ProcessExited(KProcess *);
    void ReceiveDCCChatData(KProcess *, char *, int);

/* In handleserver.cpp */
protected:
    void HandleCTCP_Reply(const char *,const char *);
    void Handle_Notice(const char *,const char *);
    void Handle_ServerPong(const char *);
    void Handle_Join(const char *,const char *,const char *);
    void Handle_Part(const char *,const char *,const char *,const char *);
    void Handle_Kick(const char *, const char *, const char *, const char *);
    void Handle_Nick(const char *, const char *);
    void Handle_Quit(const char *, const char *);
};
#endif
