/* $Id$
   kirc Module handling file, by Alex Zepeda <garbanzo@hooked.net>
 */


#include "modules.h"

#include <stdio.h>
#include <dlfcn.h>

/* Hook Entry */

/* Module Entry */

moduleEnt::moduleEnt(const char *title)
{
  name = QString(title);
  return;
}

modLoader::modLoader(){
  modules.clear();
}

/* Module Loader */

bool modLoader::load_module(const char *name)
{
  int tempid=0; unsigned int count;
  Function func; Function_Exec xfunc;
  moduleEnt *temp;
  void *handle = dlopen(name,RTLD_NOW);
  void *sym;

  if (!handle) {
    debug("dlerror: %s",dlerror());
    return false;
  }

  modules.append(new moduleEnt(name));

  sym = find_symbol(modules.count()-1,"kirc_init__Fv"); /* Ugly hack to support C++ libs */
  if (!sym) {
    debug("Couldn't load: %s (no init function)",modules.last()->name.data());
    unload_module(name);
    return false;
  }

  modules.last()->handle = sym;
  func = (Function)sym;
  tempid = (int)func();
  debug("Looking for kirc_register");
  sym = find_symbol(modules.count()-1,"kirc_register__FiPvT1");

  if (!sym){
    debug("Couldn't load: %s (no register function)",modules.last()->name.data()); 
    unload_module(name);
    return false;
  }

  xfunc = (Function_Exec)sym;
  sym = find_symbol(modules.count()-1,"kirc_exec__FPv");
  if (!sym){
    debug("Couldn't load: %s (no exec function)",modules.last()->name.data());
    unload_module(name);
    return false;
  }

  xfunc(tempid,sym,(void *)this);

  for (count=0;count<modules.count();count++) /* Check for duplicate modules */
  {
    if (modules.at(count)->id == tempid) {
      debug("Duplicate module attempt (%s).",modules.last()->name.data());
      unload_module(name);
      return false;
    }
  }
  modules.last()->id = tempid;
  modules.last()->handle = handle;
  printf("loaded module: %s\n",modules.last()->name.data());
  return true;
}

bool modLoader::unload_module(const char *name)
{
  unsigned int count;
  printf("unload_module(char*)\n");
  for (count=0;count<modules.count();count++)
    {
      if (! strcmp(modules.at(count)->name.data(),name) ){
	dlclose(modules.at(count)->handle);
	/* We've gotta destroy the hooks too */
	modules.remove(count);
	return true;
      }
    }
  return false;
}

bool modLoader::unload_module(int id)
{
  unsigned int count;
  printf("unload_module\n");
  for (count=0;count<modules.count();count++)
    {
      if (id==modules.at(count)->id)
	{
	dlclose(modules.at(count)->handle);
	/* We've gotta destroy the hooks too */
	modules.remove(count);
	return true;
      }
    }
  return false;
}

void * modLoader::find_symbol(int id,const char *symbol)
{
  void *sym;
  sym = dlsym(RTLD_NEXT,symbol);
  printf("find_symbol (%s)\n",symbol);
  if (!sym)
    debug("find_sym: %s",dlerror());
   return sym;
}

void modLoader::add_hook(Hook *ho)
{
   printf("Hook added\n");
   Hooks.append(ho);
}







